package DBomb::Version;

=head1 NAME

DBomb::Version - The DBomb Release Version

=cut

use strict;
use warnings;

## This is the RELEASE version, not the CVS Revision.
our $VERSION = '0.20'; ## tag: version-0_20 (remember to update README)

1;
__END__

