package DBomb::Test::Objects;

=head1 NAME

DBomb::Test::Objects - This is an umbrella package for all the DBomb::Test objects.

=cut

use strict;
use warnings;
our $VERSION = '$Revision: 1.3 $';


1;
__END__

