package DBomb::Value;

=head1 NAME

DBomb::Value - Abstract class for column and key values.

=cut

use strict;
use warnings;
our $VERSION = '$Revision: 1.2 $';




1;
__END__

=head1 SYNOPSIS

This class provides no functionality whatsoever.

=cut

